@@ -147,6 +147,7 @@ KWalletD::KWalletD()
 
     _dw->startScan(true);
     connect(_dw, SIGNAL(dirty(QString)), this, SLOT(emitWalletListDirty()));
+    connect(_dw, &KDirWatch::deleted, this, &KWalletD::emitWalletListDirty);
 
     _serviceWatcher.setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
     connect(&_serviceWatcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)), this, SLOT(slotServiceOwnerChanged(QString,QString,QString)));
@@ -924,7 +925,7 @@ int KWalletD::close(const QString &wallet, bool force)
     return internalClose(w, handle, force);
 }
 
-int KWalletD::internalClose(KWallet::Backend *w, int handle, bool force)
+int KWalletD::internalClose(KWallet::Backend * const w, const int handle, const bool force, const bool saveBeforeClose)
 {
     if (w) {
         const QString &wallet = w->walletName();
@@ -936,7 +937,7 @@ int KWalletD::internalClose(KWallet::Backend *w, int handle, bool force)
             }
             _syncTimers.removeTimer(handle);
             _wallets.remove(handle);
-            w->close(true);
+            w->close(saveBeforeClose);
             doCloseSignals(handle, wallet);
             delete w;
             return 0;
@@ -1482,6 +1483,12 @@ void KWalletD::emitFolderUpdated(const QString &wallet, const QString &folder)
 
 void KWalletD::emitWalletListDirty()
 {
+    const QStringList walletsInDisk = wallets();
+    foreach (auto i, _wallets.values()) {
+        if (!walletsInDisk.contains(i->walletName())) {
+            internalClose(i, _wallets.key(i), true, false);
+        }
+    }
     emit walletListDirty();
 }
 
@@ -1713,4 +1720,3 @@ int KWalletD::pamOpen(const QString &wallet, const QByteArray &passwordHash, int
     return handle;
 }
 
-#include "kwalletd.moc"
